@@ -3,7 +3,7 @@ typedload
 Module to load data into typed data structures
 """
 
-# Copyright (C) 2018-2020 Salvo "LtWorf" Tomaselli
+# Copyright (C) 2018-2021 Salvo "LtWorf" Tomaselli
 #
 # typedload is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -110,6 +110,10 @@ class Loader:
         handler. When disabled, the exceptions are not raised
         and the condition is considered False.
 
+    mangle_key: Defaults to 'name'
+        Specifies which key is used into the metadata dictionaries
+        to perform name-mangling.
+
     handlers: This is the list that the loader uses to
         perform its task.
         The type is:
@@ -127,6 +131,9 @@ class Loader:
         In most cases, it is sufficient to append new elements
         at the end, to handle more types.
 
+        There is an internal cache to speed up lookup, so after the
+        first call to load, this should no longer be modified.
+
     frefs: Dictionary to resolve ForwardRef.
         Something like
         class Node(NamedTuple):
@@ -154,21 +161,6 @@ class Loader:
     ones have any effect. This is to allow custom handlers to have their
     own parameters as well.
 
-    There is support for:
-        * Basic python types (int, str, bool, float, NoneType)
-        * NamedTuple
-        * Enum
-        * Optional[SomeType]
-        * List[SomeType]
-        * Dict[TypeA, TypeB]
-        * Tuple[TypeA, TypeB, TypeC]
-        * Tuple[SomeType, ...]
-        * Set[SomeType]
-        * Union[TypeA, TypeB]
-        * ForwardRef
-        * Literal
-        * Dataclass
-
     Using unions is complicated. If the types in the union are too
     similar to each other, it is easy to obtain an unexpected type.
     """
@@ -195,6 +187,9 @@ class Loader:
         # Enable conversion of dict-like things to dicts, before loading
         self.dictequivalence = True
 
+        # Which key is used in metadata to perform name mangling
+        self.mangle_key = 'name'
+
         # The list of handlers to use to load the data.
         # It gets iterated in order, and the first condition
         # that matches is used to load the value.
@@ -411,6 +406,25 @@ def _tupleload(l: Loader, value, type_) -> Tuple:
         return tuple(l.load(v, t, annotation=Annotation(AnnotationType.INDEX, i)) for i, (v, t) in enumerate(zip(value, args)))
 
 
+def _mangle_names(namesmap: Dict[str, str], value: Dict[str, Any]) -> Dict[str, Any]:
+    """
+    Mangling names of a dictionary.
+
+    The dictionary is copied internally.
+
+    Namesmap is the mapping to be applied, in the format [dataname] = pyname
+    """
+    if not namesmap:
+        return value
+    r = {}
+
+    for k, v in value.items():
+        if k in namesmap:
+            k = namesmap[k]
+        r[k] = v
+    return r
+
+
 def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
     """
     This loads a Dict[str, Any] into a NamedTuple.
@@ -429,20 +443,15 @@ def _namedtupleload(l: Loader, value: Dict[str, Any], type_) -> Tuple:
         #Name mangling
 
         # Prepare the list of the needed name changes
-        transforms = []  # type: List[Tuple[str, str]]
-        for field in fields:
-            if type_.__dataclass_fields__[field].metadata:
-                name = type_.__dataclass_fields__[field].metadata.get('name')
+        transforms = {}  # type: Dict[str, str]
+        for pyname in fields:
+            if type_.__dataclass_fields__[pyname].metadata:
+                name = type_.__dataclass_fields__[pyname].metadata.get(l.mangle_key)
                 if name:
-                    transforms.append((field, name))
-        # Do the needed name changes
-        if transforms:
-            value = value.copy()
-            for pyname, dataname in transforms:
-                if dataname in value:
-                    tmp = value[dataname]
-                    del value[dataname]
-                    value[pyname] = tmp
+                    transforms[name] = pyname
+
+        value = _mangle_names(transforms, value)
+
 
     necessary_fields = fields.difference(optional_fields)
     try:
@@ -583,6 +592,7 @@ def _attrload(l, value, type_):
     names = []
     defaults = {}
     types = {}
+    namesmap = {}  # type: Dict[str, str]
 
     for attribute in type_.__attrs_attrs__:
         names.append(attribute.name)
@@ -590,14 +600,11 @@ def _attrload(l, value, type_):
         defaults[attribute.name] = attribute.default
 
         # Manage name mangling
-        if 'name' in attribute.metadata:
-            dataname = attribute.metadata['name']
-            pyname = attribute.name
+        if l.mangle_key in attribute.metadata:
+            namesmap[attribute.metadata[l.mangle_key]] = attribute.name
+
+    value = _mangle_names(namesmap, value)
 
-            if dataname in value:
-                tmp = value[dataname]
-                del value[dataname]
-                value[pyname] = tmp
 
     t = _FakeNamedTuple((
         tuple(names),
